@@ -13,7 +13,7 @@ class Config(gconf.Client):
         gconf.Client.__init__(self)
         self._base_path = base_path
         self.add_dir(base_path[0:-1], gconf.CLIENT_PRELOAD_RECURSIVE)
-                
+                 
     def notify_add(self, k, f):
         return gconf.Client.notify_add(self, self._base_path + k, f)
 
@@ -22,7 +22,7 @@ class Config(gconf.Client):
     
     def __contains__(self, k):
         return self.get(self._base_path + k) is not None
-    
+        
     def __getitem__(self, k):
         try:
             return self.get_value(self._base_path + k)
@@ -36,5 +36,7 @@ class Config(gconf.Client):
         """
         if v is None:
             self.unset(self._base_path + k)
+        elif isinstance(v, list):
+            self.set_list(self._base_path+k, gconf.VALUE_STRING, [str(e) for e in v])
         else:
             self.set_value(self._base_path + k, v)
