@@ -25,8 +25,8 @@ class Player(GObject):
     def __init__(self, config):
         GObject.__init__(self)
         self.playing = False
-        self.playlist = Playlist()
-        self.jump_to = None
+        self.playlist = None
+
         self._current = None
         self._player = None
         self._seek_emit_id = None
@@ -38,36 +38,28 @@ class Player(GObject):
         self._config.notify_add("rg-preamp", lambda client, cxn_id, entry, data: self._apply_volume())
         self._config.notify_add("no-rg-preamp", lambda client, cxn_id, entry, data: self._apply_volume())
     
-    def next(self, *args):
-        if self.jump_to is not None:
-            self.goto_pos(self.playlist.index(self.jump_to))
-            self.jump_to = None
+    def next(self, play=True, *args):
+        if self.playlist.jump_to is not None:
+            self.goto_pos(self.playlist.jump_index(), play)
+            self.playlist.jump_to = None
         else:
-            same_playlist = False
-            
-            try:
-                same_playlist = self._current is self.playlist[self._config["pos"]]
-            except IndexError:
-                pass
-            
-            if same_playlist:
-                self.goto_pos(self._config["pos"] + 1)
-            else:
-                self.goto_pos(self._config["pos"])
+            self.goto_pos(self.playlist.next_song(self._current), play)
 
     def tracks(self):
         return len(self.playlist)
 
-    def goto_pos(self, pos):
+    def goto_pos(self, pos, play=True):
         # emit song-played for previous track
         if self._current is not None:
             self._emit_played()
+        
+        pos %= len(self.playlist)
+        self._current = self.playlist[pos]
 
-        self._config["pos"] = pos % len(self.playlist)
-        # FIXME: maemo does not see the update otherwise
-        self._config.notify("pos") 
-        self._current = self.playlist[self._config["pos"]] 
-        self.start_playback()
+        self.playlist.pos = pos
+        
+        if play: 
+            self.start_playback()
 
     def previous(self):
         if self.get_time() > 3:
@@ -75,12 +67,12 @@ class Player(GObject):
             self.seek_to(0.0)
             return
 
-        self.goto_pos(self._config["pos"] - 1)
+        self.goto_pos(self.playlist.pos - 1)
     
     def get_time(self):
         """returns position in seconds"""
         try:
-            s = self._player.query_position(gst.FORMAT_TIME)[0]/ gst.SECOND
+            s = self._player.query_position(gst.FORMAT_TIME)[0] / gst.SECOND
             return s
         except gst.QueryError, e:
             sys.stderr.write("get_time: %s\n" % e)
@@ -93,7 +85,7 @@ class Player(GObject):
             sys.stderr.write("get_seek: %s\n" % e)
             return 0.0
         else:            
-            return self._player.query_position(gst.FORMAT_TIME)[0]/length
+            return self._player.query_position(gst.FORMAT_TIME)[0] / length
 
     def seek_to(self, pos):
         try:
@@ -101,14 +93,14 @@ class Player(GObject):
         except gst.QueryError, e:
             sys.stderr.write("seek_to: %s\n" % e)
         else:        
-            self._player.seek_simple(gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH, int(length*pos))
+            self._player.seek_simple(gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH, int(length * pos))
     
     def _emit_played(self):
         self.emit("song-played", self._current)
             
     def load_track(self):
         if self._current is None:
-            self._current = self.playlist[self._config["pos"]]
+            self._current = self.playlist[self.playlist.pos]
         
         if self._player is not None:
             self._player.set_state(gst.STATE_NULL)
@@ -127,7 +119,7 @@ class Player(GObject):
         bus.connect("message::error", self._on_error)
         bus.connect("message::state-changed", self._on_state_changed)
 
-        self._player.set_property("uri", self._current.uri)
+        self._player.set_property("uri", "file://"+self._current.uri)
         self._player.set_state(gst.STATE_PAUSED)
 
     def _on_state_changed(self, bus, message):
@@ -145,7 +137,7 @@ class Player(GObject):
                 self.emit("song-changed", self._current)
     
     def _set_duration(self):
-         self._current["duration"] = int(self._player.query_duration(gst.FORMAT_TIME)[0]/gst.SECOND)
+         self._current["duration"] = int(self._player.query_duration(gst.FORMAT_TIME)[0] / gst.SECOND)
          return False
    
     def start_playback(self):
@@ -177,7 +169,7 @@ class Player(GObject):
         self._vol = vol * MAX_VOL        
         
         self._apply_volume()
-		
+        
     def toggle(self):
         if self.playing:
             self._player.set_state(gst.STATE_PAUSED)
@@ -193,7 +185,7 @@ class Player(GObject):
 
     def _on_buffering(self, bus, message):
         print message.structure["buffer-percent"]
-	
+    
     def _update_song(self, new_tags):        
         for key in new_tags.keys():
             v = new_tags[key]
@@ -225,12 +217,12 @@ class Player(GObject):
                 rg_value = float(cur["replaygain-album-gain"])
             else:
                 rg_value = float(cur["replaygain-track-gain"])
-        		
+                
             rg_value += self._config["rg-preamp"]
         else:
             rg_value = self._config["no-rg-preamp"]
 
-        return pow(10, rg_value/20.0) # rg_scale
+        return pow(10, rg_value / 20.0) # rg_scale
 
     def _apply_volume(self):
         vol = self._vol
@@ -242,6 +234,10 @@ class Player(GObject):
 
     def _emit_seek(self):
         # do not send update if we are already paused
-        self.emit("seek-changed", self.get_seek())
+        try:
+            self.emit("seek-changed", self.get_seek())
+        except gst.QueryError:
+            # dont hiccup
+            pass
         
         return True # if True will be called repeatedly
