@@ -3,7 +3,7 @@ import pango
 import hildon
 
 from youamp.ui.browser import Browser
-from youamp.ui.playlist import PlaylistView
+from youamp.ui.playlist import PlaylistLabel, SongList
 from youamp.ui import ARTIST_SELECTED, ALBUM_SELECTED
 
 class BrowseButton(gtk.Button):
@@ -21,32 +21,55 @@ class BrowseButton(gtk.Button):
     def set_text(self, txt):
         self.label.set_text(txt)
 
-class SearchView(gtk.VBox):
-    def __init__(self, config, player, library):
-        gtk.VBox.__init__(self)
-        self.set_spacing(5)
+class SearchView(SongList):
+    SINK = SongList.SINK[0:1] # dont allow adding to library
+
+    def __init__(self, playlist, player, library, config, song_menu, xml):
+        self.view = gtk.VBox()
+        self.view.playlist = playlist
+        self.view.restore = super(SearchView, self).restore
+        self.view.top = self
+
+        SongList.__init__(self, playlist, player, library, song_menu)
 
         self._config = config
-        self._library = library
-        self._player = player
-        self._is_browser = self._config["is-browser"]
+        self._is_browser = config["is-browser"]
         
+        self.label = PlaylistLabel(playlist, icon="qgn_medi_library")
+
         # Navi Controls
+        self.view.set_spacing(5)
+
         navi = gtk.HBox()
         navi.set_spacing(5)
+        self.view.pack_start(navi, expand=False)
 
+        hbox = gtk.HBox()
+        navi.pack_start(hbox, expand=False)
+        
         # Search Label
         self._search_label = gtk.Label(_("Search"))
         self._search_label.set_size_request(100, -1)
-        navi.pack_start(self._search_label, expand=False)
+        hbox.pack_start(self._search_label, expand=False)
         
         # Browse Button
         self._browse_button = gtk.Button(_("Browse"))
         self._browse_button.connect("clicked", self._show_artists)
         self._browse_button.set_size_request(100, -1)
         self._browse_button.unset_flags(gtk.CAN_FOCUS)
-        navi.pack_start(self._browse_button, expand=False)
-        
+        hbox.pack_start(self._browse_button, expand=False)
+
+        arrow = gtk.Button()
+        arrow.set_relief(gtk.RELIEF_NONE)
+        arrow.add(gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_IN))
+        arrow.unset_flags(gtk.CAN_FOCUS)
+        hbox.pack_start(arrow, expand=False)
+
+        m = xml.get_object("view_menu")
+        m.attach_to_widget(arrow, lambda *args: None)
+
+        arrow.connect("clicked", self._view_menu_pop, m)
+
         # Navi Artist Button
         self._artist = BrowseButton()
         self._artist.connect("clicked", self._show_albums)
@@ -65,28 +88,25 @@ class SearchView(gtk.VBox):
         
         # Shuffle Button
         shuffle = gtk.ToggleButton(_("shuffle"))
-        shuffle.unset_flags(gtk.CAN_FOCUS)
         shuffle.set_active(self._config["shuffle"])
+        shuffle.unset_flags(gtk.CAN_FOCUS)
         self._shndl = shuffle.connect("toggled", self._on_shuffle_toggled)
         self._shuffle = shuffle
         
         navi.pack_end(shuffle, expand=False)
-        self.pack_start(navi, expand=False)
         
         # Scrolled View
         self._scroll = gtk.ScrolledWindow()
         self._scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
         self._scroll.set_shadow_type(gtk.SHADOW_IN)
         hildon.hildon_helper_set_thumb_scrollbar(self._scroll, True)
+        self.view.pack_start(self._scroll)
         
-        self._playlist = PlaylistView(player.playlist, self._config, self._player)        
-        self._browser = Browser(self._config, self._library)
+        self._browser = Browser(config, library)
         
-        self._scroll.add(self._playlist)
+        self._scroll.add(self)
 
-        self.pack_start(self._scroll)
-
-        self.show_all()
+        self.view.show_all()
 
         # browser callbacks
         self._config.notify_add("search-artist", self._on_artist_changed)
@@ -94,7 +114,13 @@ class SearchView(gtk.VBox):
         
         self._config.notify_add("is-browser", self._on_view_changed)
         self._config.notify_add("shuffle", self._update_shuffle_btn)
-        
+
+    def _view_menu_pop(self, button, m):
+        a = button.get_allocation()
+        ap = button.get_parent_window().get_position()
+
+        m.popup(None, None, lambda *arg: (a.x + ap[0], a.y + a.height + ap[1], False), 0, 0)
+     
     def _update_shuffle_btn(self, client, cxn_id, entry, data):
         self._shuffle.handler_block(self._shndl)
         self._shuffle.set_active(entry.get_value().get_bool())
@@ -137,14 +163,15 @@ class SearchView(gtk.VBox):
             self._browse_button.hide()
             self._artist.hide()
             self._album.hide()
-        
-        self._playlist.restore()
+
+        SongList.restore(self)
             
     def _on_shuffle_toggled(self, caller):
-        self._config["shuffle"] = not self._config["shuffle"]
-        self._player.playlist.shuffle(self._config["shuffle"])
+        new_shuffle_state = not self._config["shuffle"]
+        self._config["shuffle"] = new_shuffle_state
         
-        self._config["pos"] = self._player.playlist.get_new_pos(self._config["pos"])
+        self._model.shuffle(new_shuffle_state)
+        self.set_cursor(self._model.pos)
         
     def _browse_complete(self):        
         self._search_entry.set_text("")
@@ -171,13 +198,13 @@ class SearchView(gtk.VBox):
             self._show_playlist()
     
     def _show_playlist(self):
-        if self._playlist.get_parent() is None:
+        if self.get_parent() is None:
             self._scroll.remove(self._browser)
-            self._scroll.add(self._playlist)
+            self._scroll.add(self)
 
     def _show_browser(self):
         if self._browser.get_parent() is None:
-            self._scroll.remove(self._playlist)
+            self._scroll.remove(self)
             self._scroll.add(self._browser)
 
     def _on_artist_changed(self, client, cxn_id, entry, data):
